#ifndef _DRV_TOUCH_H_
#define _DRV_TOUCH_H_

#define TOUCH_SAMPLE_PREVIOUSDOWN       (1<<8)
#define TOUCH_SAMPLE_CALIBRATED         (1<<9)

#define TOUCH_SAMPLE_IGNORE             (1<<0)
#define TOUCH_SAMPLE_VALID              (1<<1)
#define TOUCH_SAMPLE_DOWN               (1<<2)

// for calibaration
#define     MAX_NCALIBRATION        (5)

// Scale factor to support sub-pixel resolutions
#define X_SCALE_FACTOR      4
#define Y_SCALE_FACTOR      4

//===========================================================================
//#define __7INCH_DISPLAY__
#ifdef __7INCH_DISPLAY__
    #define DISPLAY_WIDTH       800
    #define DISPLAY_HEIGHT      480
#else
    #define DISPLAY_WIDTH       800
    #define DISPLAY_HEIGHT      600
#endif
//===========================================================================

#define TSP_LCDX					    (DISPLAY_WIDTH * X_SCALE_FACTOR)
#define TSP_LCDY						(DISPLAY_HEIGHT * Y_SCALE_FACTOR)

/** Touchscreen ADC clock frequency to use. */
#define BOARD_TOUCHSCREEN_ADCCLK    300000 /* 8MHz max */
/** Touchscreen ADC startup time in seconds. */
#define BOARD_TOUCHSCREEN_STARTUP   40
/** Touchscreen ADC track and hold time in nanoseconds. */
#define BOARD_TOUCHSCREEN_SHTIM     20000    /* min 1s at 8MHz */
/** Touchscreen pen debounce time in nanoseconds. */
#define BOARD_TOUCHSCREEN_DEBOUNCE  10000000

#define BOARD_MAINOSC           12000000

/** Master clock frequency (when using board_lowlevel.c) */
#define BOARD_MCK                ((unsigned long)((BOARD_MAINOSC / 3 / 2 / 3) * 200 ))

/** Status that used for touchscreen */
#define TS_STATUSES ( ADC_ISR_PENS | ADC_ISR_PEN | ADC_ISR_NOPEN \
                    | ADC_ISR_XRDY | ADC_ISR_YRDY | ADC_ISR_PRDY )

/** X value is ready */
#define TS_X_RDY        (1 << 0)
/** Y value is ready */
#define TS_Y_RDY        (1 << 1)
/** Pressure value is ready */
#define TS_P_RDY        (1 << 2)
/** Pen status */
#define TS_PEN_STAT     (1 << 7)
/** All data is ready (X,Y & P) */
#define TS_DATA_RDY     (TS_X_RDY|TS_Y_RDY)//|TS_P_RDY)
					
EXTERN UINT32   HAL_TouchInit( VOID );
EXTERN VOID     HAL_TouchGetPoint( PUINT32 pfTipState, INT32 *pUncalX, INT32 *pUncalY);

EXTERN BOOL     TouchCalibration( PINT32 pXscr, PINT32 pYscr, PINT32 pXtch, PINT32 pYtch);
EXTERN VOID     TouchCalibrate( INT32 Xtch, INT32 Ytch, INT32 *pXscr, INT32 *pYscr);


#endif // _DRV_TOUCH_H_
