/*	$ DOOROOS : touch.c, v 0.1 2013/09/01 $	*/

/*
 * Copyright (C) 2013	Jung Byong-Oh
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: touch.c
 *
 * Purpose: touch pad interface for graphic system of j-bosn
 */

#include <types.h>
#include <thread.h>
#include <message.h>
#include <interrupt.h>
#include <device.h>
#include <error.h>
#include <fs.h>

#include <window.h>

#include <mach/hal_interrupt.h>

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3Xbd.h>
#include <mach/gpio.h>

#include <hal_panic.h>

//#include <mach/spi.h>

#include "touch.h"


#define DBG_LVL_TOUCH		0
#define TOUCH_TEST			0
//--------------------------------------------------------------
// Defines
//--------------------------------------------------------------

// Sample filters
#define MAX_DIFFERENCE_CHANGE		100
#define SAMPLING_TIME				0x3FFF

static SHARED_MEMORY * pSharedMemory = (SHARED_MEMORY *)SHARED_MEMORY_BASE;

DWORD gIntrTouch        = 0;
STATIC UINT32 ADCClock = 0;
//// AT91 Peripherals
//STATIC AT91PS_PIO pPIO_PENIRQ = NULL;
//STATIC AT91PS_PIO pPIOA_Base = (AT91PS_PIO)AT91C_BASE_PIOA;

#define ABS(a)              ((a)>=0 ? (a): -(a))
STATIC UINT32 RotateAngle = 0;

DEVICECONFIG            TCH_DeviceConfig ={"TCH", 0, -1 };
//===========================================================================
// this value is define the resolution of touch ... MUST define

#define TOUCH_WIDTH         DISPLAY_WIDTH
#define TOUCH_HEIGHT        DISPLAY_HEIGHT

INT32       Xscr[MAX_NCALIBRATION] ={TOUCH_WIDTH/2,   TOUCH_WIDTH/5,  TOUCH_WIDTH/5,    TOUCH_WIDTH*4/5,   TOUCH_WIDTH*4/5};
INT32       Yscr[MAX_NCALIBRATION] ={TOUCH_HEIGHT/2,  TOUCH_HEIGHT/5, TOUCH_HEIGHT*4/5, TOUCH_HEIGHT*4/5,  TOUCH_HEIGHT/5};

INT32       Xscr_90[MAX_NCALIBRATION] ={TOUCH_HEIGHT/2,  TOUCH_HEIGHT/5, TOUCH_HEIGHT/5, TOUCH_HEIGHT*4/5,  TOUCH_HEIGHT*4/5};
INT32       Yscr_90[MAX_NCALIBRATION] ={TOUCH_WIDTH/2,   TOUCH_WIDTH/5,  TOUCH_WIDTH*4/5,    TOUCH_WIDTH*4/5,   TOUCH_WIDTH/5};


#ifdef __7INCH_DISPLAY__
// ibosn h/w
//INT32       Xtch[MAX_NCALIBRATION] ={0x1f9,  0x313,  0x31b, 0x0d9,  0x0e0};
//INT32       Ytch[MAX_NCALIBRATION] ={0x206,  0x303,  0x0f7, 0x0f9,  0x302};
//mpiece h/w
//INT32       Xtch[MAX_NCALIBRATION] ={0x204,  0x0F8,  0x31A, 0x312,  0x0FB};
//INT32       Ytch[MAX_NCALIBRATION] ={0x203,  0x0DC,  0x0D5, 0x328,  0x324};
INT32       Xtch[MAX_NCALIBRATION] ={0x1FC,  0x0DF,  0x0dd, 0x323,  0x328};
INT32       Ytch[MAX_NCALIBRATION] ={0x20C,  0x0FD,  0x310, 0x310,  0x0F1};
#else

#ifdef __TOUCH_REVERSE__
//INT32       Xtch[MAX_NCALIBRATION] ={0x217,  0x0fa,  0x0f5,  0x32a,  0x330};
//INT32       Ytch[MAX_NCALIBRATION] ={0x1fe,  0x2fb,  0x0fb,  0x0fb,  0x2fe};
//INT32       Xtch[MAX_NCALIBRATION] ={0x210,  0x2f1,  0x2ff,  0x149,  0x152};
//INT32       Ytch[MAX_NCALIBRATION] ={0x1f8,  0x0f8,  0x2f8,  0x2ed,  0x14C};

INT32       Xtch[MAX_NCALIBRATION] ={0x20f,  0x119,  0x122, 0x306,  0x30c};
INT32       Ytch[MAX_NCALIBRATION] ={0x208,  0x312,  0x108, 0x0fa,  0x30d};

INT32       Xtch_90[MAX_NCALIBRATION] ={0x208,  0x108,  0x0fa, 0x30d,  0x312};
INT32       Ytch_90[MAX_NCALIBRATION] ={0x20f,  0x122,  0x306, 0x30c,  0x119};

#else
//INT32       Xtch[MAX_NCALIBRATION] ={523,  802,  798,  253,  250};
//INT32       Ytch[MAX_NCALIBRATION] ={471,  220,  720,  713,  222};

//INT32       Xtch[MAX_NCALIBRATION] ={0x206,  0x0ed,  0x0ea,  0x328,  0x328};
//INT32       Ytch[MAX_NCALIBRATION] ={0x209,  0x318,  0x0f7,  0x0f9,  0x318};

INT32       Xtch[MAX_NCALIBRATION] ={0x208,  0x0e4,  0x0e8,  0x32d,  0x325};
INT32       Ytch[MAX_NCALIBRATION] ={0x203,  0x313,  0x0f9,  0x0f6,  0x30d};

INT32       Xtch_90[MAX_NCALIBRATION] ={0x203,  0x0f9,  0x0f6, 0x30d,  0x313};
INT32       Ytch_90[MAX_NCALIBRATION] ={0x208,  0x0e8,  0x32d, 0x325,  0x0e4};

#endif

#endif

PUBLIC UINT32 TouchDisplayWidth = TOUCH_WIDTH;
PUBLIC UINT32 TouchDisplayHeight = TOUCH_HEIGHT;

//===========================================================================

STATIC
VOID ADC_SetTsAverage(AT91PS_TSADC pADC, UINT32 Avg2Conv)
{
    UINT32 dwMr = pADC->ADC_TSMR & ~ADC_TSMR_TSAV_Msk;
    UINT32 dwTSAV = Avg2Conv >> ADC_TSMR_TSAV_Pos;
    UINT32 dwTSFREQ = (dwMr & ADC_TSMR_TSFREQ_Msk) >> ADC_TSMR_TSFREQ_Pos;
    if (dwTSAV)
    {
        if (dwTSAV > dwTSFREQ)
        {
            dwMr &= ~ADC_TSMR_TSFREQ_Msk;
            dwMr |=  ADC_TSMR_TSFREQ(dwTSAV);
        }
    }
    pADC->ADC_TSMR = dwMr | Avg2Conv;
}
STATIC
VOID ADC_SetTsDebounce(AT91PS_TSADC pADC, UINT32 dwTime)
{
    UINT32 dwDiv = 1000000000;
    UINT32 dwClk = ADCClock;
    UINT32 dwPenbc = 0;
    UINT32 dwTarget, dwCurrent;
    UINT32 dwTsmr;
	
    if (dwTime == 0 || ADCClock == 0) return;
    /* Divide time & ADCCLK to avoid overflows */
    while ((dwDiv > 1) && ((dwTime % 10) == 0))
    {
        dwTime /= 10; dwDiv /= 10;
    }
    while ((dwDiv > 1) && ((dwClk & 10) == 0))
    {
        dwClk /= 10; dwDiv /= 10;
    }
    /* Compute PENDBC */
    dwTarget = dwTime * dwClk / dwDiv;
    dwCurrent = 1;
    while (dwCurrent < dwTarget)
    {
        dwPenbc ++; dwCurrent *= 2;
    }
    dwTsmr = ADC_TSMR_PENDBC(dwPenbc);
    if (dwTsmr == 0) return;
    dwTsmr |= pADC->ADC_TSMR & ~ADC_TSMR_PENDBC_Msk;
    pADC->ADC_TSMR = dwTsmr;
}
STATIC
VOID ADC_SetTriggerMode(AT91PS_TSADC pADC, UINT32 dwMode)
{
    UINT32 dwTrgr = pADC->ADC_TRGR & ~ADC_TRGR_TRGMOD_Msk;
    pADC->ADC_TRGR = dwTrgr | dwMode;
}


/**
 * Enable/Disable TSD capturing
 */
STATIC
VOID TSD_Enable(BOOL bEnDis)
{
	AT91PS_TSADC pADC = (AT91PS_TSADC)AT91C_BASE_TSADC;
    if (bEnDis)
    {
        ADC_SetTsAverage(pADC, ADC_TSMR_TSAV_NO_FILTER);
        pADC->ADC_CR = ADC_CR_TSCALIB;
        ADC_SetTriggerMode(pADC, ADC_TRGR_TRGMOD_PEN_TRIG);
		pADC->ADC_IER = ADC_IER_PEN;
    }
    else
    {
		UINT32 tmp;
        ADC_SetTriggerMode(pADC, ADC_TRGR_TRGMOD_NO_TRIGGER);
		pADC->ADC_IDR = TS_STATUSES;
		tmp = pADC->ADC_ISR;
		tmp = pADC->ADC_PRESSR;
    }
}

PUBLIC VOID
HAL_TouchGetPoint( PUINT32 pTipState, INT32 *pUnCalX, INT32 *pUnCalY)
{
	// Sampled
	static INT iPrevSampleX	= 0;
	static INT iPrevSampleY	= 0;
	// Last valid sample value
	static INT iPrevValidSampleX	= 0;
	static INT iPrevValidSampleY	= 0;
	// Skipped samples
	//static DWORD dwSkippedSamples = 0;

	STATIC UINT32 bTsFlags = 0;
	STATIC BOOL bIsPenDown = FALSE;

    UINT32 Status;
	AT91PS_TSADC pADC = (AT91PS_TSADC)AT91C_BASE_TSADC;
	
    // Retrieve status
    Status = pADC->ADC_ISR & TS_STATUSES;
	//TSD_Enable(FALSE);

//	DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): Status = %x\r\n", __FUNCTION__, __LINE__, Status);
    
    // Contact lost
    if (Status & ADC_ISR_NOPEN)
    {
PENUP:
		/* Stop periodic trigger & enable pen */
		ADC_SetTsAverage(pADC, ADC_TSMR_TSAV_NO_FILTER);
		ADC_SetTsDebounce(pADC, BOARD_TOUCHSCREEN_DEBOUNCE);
		ADC_SetTriggerMode(pADC, ADC_TRGR_TRGMOD_PEN_TRIG);

		pADC->ADC_IDR = ADC_IDR_NOPEN|ADC_IER_XRDY|ADC_IER_YRDY|ADC_IER_PRDY;
		pADC->ADC_IER = ADC_IER_PEN;

		bIsPenDown = FALSE;

		bTsFlags = 0;
		// Set flags
		*pTipState = TOUCH_SAMPLE_VALID;
		DEBUGMSG(DBG_LVL_TOUCH,"TouchPanelGetPoint : PenUp detected.\r\n");

		// Return the last valid sample
		*pUnCalX = iPrevValidSampleX;
		*pUnCalY = iPrevValidSampleY;

		// Reset values
		iPrevSampleX = 0;
		iPrevSampleY = 0;
		//dwSkippedSamples = 0;
    }
    // Pen detected
    else
	if(Status & (ADC_ISR_PEN|ADC_ISR_PENS))
	{
		bIsPenDown = TRUE;
		if(Status & ADC_ISR_PEN)
		{

	DEBUGMSG(DBG_LVL_TOUCH,"TouchPanelGetPoint : PenDown detected.\r\n");
			/* Configure for peripdic trigger */
			//ADC_SetTsAverage(pADC, ADC_TSMR_TSAV_AVG8CONV);
			ADC_SetTsAverage(pADC, ADC_TSMR_TSAV_NO_FILTER);
			ADC_SetTsDebounce(pADC, 300);         /* 300ns */
			ADC_SetTriggerMode(pADC, ADC_TRGR_TRGMOD_PERIOD_TRIG);

			pADC->ADC_IDR = ADC_IDR_PEN;
			pADC->ADC_IER = ADC_IER_NOPEN|ADC_IER_XRDY|ADC_IER_YRDY;//|ADC_IER_PRDY;
			
			*pTipState = TOUCH_SAMPLE_IGNORE;
		}
		
		if ((Status & ADC_ISR_PENS))
		{
			/* X */
			if (Status & ADC_ISR_XRDY)
			{
				bTsFlags |= TS_X_RDY;
			}
			/* Y */
			if (Status & ADC_ISR_YRDY)
			{
				bTsFlags |= TS_Y_RDY;
			}
			/* P: (X/1024)*[(Z2/Z1)-1] */
/*			if (Status & ADC_ISR_PRDY)
			{
				bTsFlags |= TS_P_RDY;
			}
			*/
			if (((bTsFlags & TS_DATA_RDY) == TS_DATA_RDY))
			{
				UINT32 X, Y;
				UINT32 tmpX, tmpY;
				//UINT32 Z = pADC->ADC_PRESSR;
				bTsFlags &= ~TS_DATA_RDY;
		
				if( bIsPenDown )
				{
					bTsFlags &= ~TS_PEN_STAT;
					
					tmpY = pADC->ADC_XPOSR;
					tmpX = pADC->ADC_YPOSR;
DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): (%8.8x , %8.8x)\r\n", __FUNCTION__, __LINE__, tmpX, tmpY);

					if (((tmpX & 0xFFF)  <= 0) || ((tmpY & 0xFFF) <= 0))
						goto PENUP;

					X = ((tmpX & ADC_XPOSR_XPOS_Msk) >> ADC_XPOSR_XPOS_Pos) << 10;//* 1024;
					X /= ((tmpX & ADC_XPOSR_XSCALE_Msk) >> ADC_XPOSR_XSCALE_Pos);

					Y = ((tmpY & ADC_YPOSR_YPOS_Msk) >> ADC_YPOSR_YPOS_Pos) << 10; //* 1024;
					Y /= ((tmpY & ADC_YPOSR_YSCALE_Msk) >> ADC_YPOSR_YSCALE_Pos);


//DBGLINE(1,"pSharedMemory->RotateAngle != RotateAngle = %d %d\r\n", pSharedMemory->RotateAngle, RotateAngle);
                    if( pSharedMemory->RotateAngle != RotateAngle )
                    {
                        RotateAngle = pSharedMemory->RotateAngle;

                        // TODO : recalibrarion
                        if( RotateAngle )
                        {
                            TouchDisplayWidth = TOUCH_HEIGHT;
                            TouchDisplayHeight = TOUCH_WIDTH;
                            TouchCalibration( Xscr_90, Yscr_90, Xtch_90, Ytch_90);
                        }
                        else
                        {
                            TouchDisplayWidth = TOUCH_WIDTH;
                            TouchDisplayHeight = TOUCH_HEIGHT;
                            TouchCalibration( Xscr, Yscr, Xtch, Ytch);
                        }
                    }

                    {
                        if( RotateAngle )
                        {
                            *pUnCalX = (INT32)Y;
                            *pUnCalY = (INT32)X;
                        }
                        else
                        {
                            *pUnCalX = (INT32)X;
                            *pUnCalY = (INT32)Y;
                        }
                    }
/*
#ifdef __7INCH_DISPLAY__
#ifdef __TOUCH_REVERSE__
                    *pUnCalX = (INT32)Y;
					*pUnCalY = (INT32)X;
#else
                    *pUnCalX = (INT32)X;
					*pUnCalY = (INT32)Y;
#endif
#else
#ifdef __TOUCH_REVERSE__
                    *pUnCalX = (INT32)Y;
					*pUnCalY = (INT32)X;
#else
                    *pUnCalX = (INT32)X;
					*pUnCalY = (INT32)Y;
#endif

#endif
*/
//DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): (x=%x , y= %x)\r\n", __FUNCTION__, __LINE__, *pUnCalX, *pUnCalY);
//DEBUGMSG(1,"%s (%d): (x=%x , y= %x)\r\n", __FUNCTION__, __LINE__, *pUnCalX, *pUnCalY);
					// Skip erroneous samples 
					if ((iPrevSampleX != 0 && iPrevSampleY != 0) &&
						((ABS(iPrevSampleX - *pUnCalX) > MAX_DIFFERENCE_CHANGE) ||
						 (ABS(iPrevSampleY - *pUnCalY) > MAX_DIFFERENCE_CHANGE)) )
					{
						*pTipState =  TOUCH_SAMPLE_IGNORE;
//DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): (x=%d , y= %d)\r\n", __FUNCTION__, __LINE__, *pUnCalX, *pUnCalY);

					}
					else
					{
						*pTipState =  TOUCH_SAMPLE_VALID | TOUCH_SAMPLE_DOWN;
			//			*pTipState = TOUCH_SAMPLE_IGNORE;
//DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): (x=%d , y= %d)\r\n", __FUNCTION__, __LINE__, *pUnCalX, *pUnCalY);
//DEBUGMSG(1,"%s (%d): (x=%x , y= %x)\r\n", __FUNCTION__, __LINE__, *pUnCalX, *pUnCalY);
						// Store the sample for futur use
						iPrevValidSampleX = *pUnCalX;
						iPrevValidSampleY = *pUnCalY;

                        /* corner detection : left top*/
                        if( X < Xtch[1] &&
                            Y > Ytch[1] )
                        {
                            pSharedMemory->TouchBootCorner = 1<<0;
                        }

                        /* corner detection : right top */
                        if( X > Xtch[4] &&
                            Y > Ytch[4] )
                        {
                            pSharedMemory->TouchBootCorner = 1<<1;
                        }
					}

					// Store the sample even if it as been skipped
					iPrevSampleX = *pUnCalX;
					iPrevSampleY = *pUnCalY;
				}
				else
				{
					/*
					pADC->TSADC_IDR = AT91C_TSADC_EOC3 | AT91C_TSADC_NOCNT;
					pADC->TSADC_IER = AT91C_TSADC_PENCNT;
					*/
				}
			}
		}
	}
    else
    {
		//DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): TouchPanelGetPoint : Not Matched.\r\n", __FUNCTION__, __LINE__);

		bIsPenDown = FALSE;

		// Set flags
		*pTipState = TOUCH_SAMPLE_IGNORE;
		// Reset values
		iPrevSampleX = 0;
		iPrevSampleY = 0;
		//dwSkippedSamples = 0;
    }

    pSharedMemory->InputCount++;
	InterruptDone(gIntrTouch);
}



PUBLIC UINT32
HAL_TouchInit( VOID )
{
	UINT32	dwMr;
	AT91PS_TSADC pADC = (AT91PS_TSADC)AT91C_BASE_TSADC;
	UINT32	i;
/*	
    const struct pio_desc hw_pio[] = {
        {"ADTRG",      AT91C_PIN_PB(18),   0, PIO_PULLUP, PIO_PERIPH_B},
    };

    pio_setup(hw_pio, sizeof(hw_pio)/sizeof(struct pio_desc));
*/

    DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );

    // Configure touchscreen
    // Disable pull-ups on touchscreen pins (pins are configured automatically)
//    AT91C_BASE_PIOA->PIO_PPUDR = 0x001E0000;

    // Enable peripheral clock
    AT91C_BASE_PMC->PMC_PCER0 = 1 << SAMA5D3_ID_ADC;
/*
	for(i=0;i<5;i++)	//SET ADC channel for touch
	{
		pADC->ADC_CHDR |= (1 << i);
	}
*/

{
#define FILE_NAME_CAL   "/nand/tchcal.bin"
//#define FILE_NAME_CAL   "/nnd/tchcal.bin"
    //read calibration info
    HANDLE hFile = FsOpen( (BYTE*)FILE_NAME_CAL );
    if( hFile != NULL )
    {
        FsRead( hFile, (BYTE*)Xtch, sizeof(int)*5);
        FsRead( hFile, (BYTE*)Ytch, sizeof(int)*5);
        FsClose( hFile );
    }
}

    DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );
	{
		UINT32	dwPres;
		/* Formula for PRESCAL is:
		   PRESCAL = (MCK / (2 * ADCCLK)) + 1
		   First, we do the division, multiplied by 10 to get higher precision
		   If the last digit is not zero, we round up to avoid generating a higher
		   than required frequency. */
		dwPres = (BOARD_MCK * 5) / BOARD_TOUCHSCREEN_ADCCLK;
		if (dwPres % 10) dwPres = dwPres / 10;
		else
		{
			if (dwPres == 0) return 0;
			dwPres = dwPres / 10 - 1;
		}

		dwMr = ADC_MR_PRESCAL(dwPres);
		if (dwMr == 0) return 0;

		dwMr |= (pADC->ADC_MR & ~ADC_MR_PRESCAL_Msk);
		pADC->ADC_MR = dwMr;
		ADCClock = BOARD_MCK / (dwPres + 1) / 2;
DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): pADC->ADC_MR = 0x%x\n", __FUNCTION__, __LINE__, pADC->ADC_MR);
	}	
    DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );

	{
		UINT32 dwStart;

		if (ADCClock == 0) return 0;
		/* Formula for STARTUP is:
		   STARTUP = (time x ADCCLK) / (1000000) - 1
		   Division multiplied by 10 for higher precision */
		
		dwStart = (BOARD_TOUCHSCREEN_STARTUP * ADCClock) / (100000);
		if (dwStart % 10) dwStart /= 10;
		else
		{
			dwStart /= 10;
			if (dwStart) dwStart --;
		}
		if      (dwStart >  896) dwMr = ADC_MR_STARTUP_SUT960;
		else if (dwStart >  832) dwMr = ADC_MR_STARTUP_SUT896;
		else if (dwStart >  768) dwMr = ADC_MR_STARTUP_SUT832;
		else if (dwStart >  704) dwMr = ADC_MR_STARTUP_SUT768;
		else if (dwStart >  640) dwMr = ADC_MR_STARTUP_SUT704;
		else if (dwStart >  576) dwMr = ADC_MR_STARTUP_SUT640;
		else if (dwStart >  512) dwMr = ADC_MR_STARTUP_SUT576;
		else if (dwStart >  112) dwMr = ADC_MR_STARTUP_SUT512;
		else if (dwStart >   96) dwMr = ADC_MR_STARTUP_SUT112;
		else if (dwStart >   80) dwMr = ADC_MR_STARTUP_SUT96;
		else if (dwStart >   64) dwMr = ADC_MR_STARTUP_SUT80;
		else if (dwStart >   24) dwMr = ADC_MR_STARTUP_SUT64;
		else if (dwStart >   16) dwMr = ADC_MR_STARTUP_SUT24;
		else if (dwStart >    8) dwMr = ADC_MR_STARTUP_SUT16;
		else if (dwStart >    0) dwMr = ADC_MR_STARTUP_SUT8;
		else                     dwMr = ADC_MR_STARTUP_SUT0;
		dwMr |= pADC->ADC_MR & ~ADC_MR_STARTUP_Msk;
		pADC->ADC_MR = dwMr;
DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): pADC->ADC_MR = 0x%x\n", __FUNCTION__, __LINE__, pADC->ADC_MR);
	}
    DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );

	{
		UINT32	dwShtim;

		if (ADCClock == 0) return 0;
		/* Formula for SHTIM is:
		   SHTIM = (time x ADCCLK) / (1000000000) - 1
		   Since 1 billion is close to the maximum value for an integer, we first
		   divide ADCCLK by 1000 to avoid an overflow */
		dwShtim = (BOARD_TOUCHSCREEN_SHTIM * (ADCClock / 1000)) / 100000;
		if (dwShtim % 10) dwShtim /= 10;
		else
		{
			dwShtim /= 10;
			if (dwShtim) dwShtim --;
		}
		dwMr  = ADC_MR_TRACKTIM(dwShtim);
		dwMr |= pADC->ADC_MR & ~ADC_MR_TRACKTIM_Msk;
		pADC->ADC_MR = dwMr;
DEBUGMSG(DBG_LVL_TOUCH+1,"%s (%d): pADC->ADC_MR = 0x%x\n", __FUNCTION__, __LINE__, pADC->ADC_MR);
	}

    DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );
	{
		UINT32 dwTrgper;
		UINT32 dwDiv = 100000000;
		UINT32 dwTrgr;
		UINT32 dwPeriod = 20000000;
		if (ADCClock == 0) return 0;
		while (dwPeriod >= 10 && dwDiv >= 10)
		{
			dwPeriod /= 10; dwDiv /= 10;
		}
		dwTrgper = (dwPeriod * ADCClock) / dwDiv;
		if (dwTrgper % 10) dwTrgper /= 10;
		else
		{
			dwTrgper /= 10;
			if (dwTrgper) dwTrgper --;
		}
		dwTrgr = ADC_TRGR_TRGPER(dwTrgper);
		dwTrgr |= pADC->ADC_TRGR & ~ADC_TRGR_TRGPER_Msk;
		pADC->ADC_TRGR = dwTrgr;
DEBUGMSG(DBG_LVL_TOUCH+1,"%s (%d): pADC->ADC_TRGR= 0x%x\n", __FUNCTION__, __LINE__, pADC->ADC_TRGR);
	}

DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );
	pADC->ADC_TSMR = (pADC->ADC_TSMR & ~ADC_TSMR_TSMODE_Msk) | ADC_TSMR_TSMODE_4_WIRE_NO_PM;

	ADC_SetTsAverage(pADC, ADC_TSMR_TSAV_NO_FILTER);

	pADC->ADC_TSMR |=  ADC_TSMR_PENDET;
	
	ADC_SetTsDebounce(pADC, BOARD_TOUCHSCREEN_DEBOUNCE);

DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );
	//===============================================================================

	
	ADC_SetTsAverage(pADC, ADC_TSMR_TSAV_NO_FILTER);
	ADC_SetTriggerMode(pADC, ADC_TRGR_TRGMOD_PEN_TRIG);
	pADC->ADC_CR = ADC_CR_TSCALIB;
	pADC->ADC_IER = ADC_IER_PEN;
    
DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );
	{
		SYSINTR_REQUEST     SysIntrRequest;
		UINT32              nSysIntr = 0;


		// request sysintr    
		SysIntrRequest.nHalIrq = SAMA5D3_ID_ADC;
		SysIntrRequest.BitMap = 1<<0;
		SysIntrRequest.Flag = SYSINTR_REQ_FLAG_STATIC;

		if( !KernelIoControl( IOCTRL_HAL_SYSINTR_REQUEST, &SysIntrRequest, sizeof( SYSINTR_REQUEST ), &nSysIntr, sizeof(UINT32) ) )
		{
			DEBUGMSG( DBG_LVL_TOUCH,"%s (%d): Error, nSysIntr = %d\n", __FUNCTION__, __LINE__, nSysIntr );
			return 0;
		}
		DEBUGMSG( DBG_LVL_TOUCH,"%s (%d): nSysIntr = %d\n", __FUNCTION__, __LINE__, nSysIntr);
		gIntrTouch = nSysIntr;
	}
DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );
#if TOUCH_TEST
	for(;;)
	{
		DEBUGMSG( DBG_LVL_TOUCH,"%s (%d): X = %d, Y = %d, pADC->ADC_ISR = %X\n", __FUNCTION__, __LINE__, pADC->ADC_XPOSR,pADC->ADC_YPOSR, pADC->ADC_ISR);
		DEBUGMSG( DBG_LVL_TOUCH,"%s (%d): ADC_CHSR = %X,ADC_TSMR = %X\n", __FUNCTION__, __LINE__, pADC->ADC_CHSR,pADC->ADC_TSMR);
		DEBUGMSG( DBG_LVL_TOUCH,"%s (%d): ADC_CDR0 = %X,ADC_CDR1 = %X,ADC_CDR2 = %X,ADC_CDR3 = %X\n", __FUNCTION__, __LINE__, pADC->ADC_CDR[0],pADC->ADC_CDR[1],pADC->ADC_CDR[2],pADC->ADC_CDR[3]);
		Sleep(100);
	}
#endif
DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );
	return gIntrTouch;
}



BOOL
HAL_TouchControl( UINT32 Cmd, PVOID pInBuf, PVOID pOutBuf )
{
	return TRUE;
}


//#define __USE_POWER_CONTROL__
UINT32
HAL_TouchPowerOff( VOID )
{
	DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );


	DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): --\n", __FUNCTION__, __LINE__ );

	return 0;
}

UINT32
HAL_TouchPowerOn( VOID )
{
	DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): ++\n", __FUNCTION__, __LINE__ );


	DEBUGMSG(DBG_LVL_TOUCH,"%s (%d): --\n", __FUNCTION__, __LINE__ );

	return 0;
}



