#include	<types.h>

#include 	"ADS7843.h"



//-----------------------------------------------------------------------------
//!
//! \brief		This function performs the SPI transaction for ADS7843
//!
//! \param		pOpenContext		SPI Device	
//! \param		ucCmd	: Command to send to ADS7843
//! \param		pBufOut : Pointer to the buffer used to transfer the output data from 
//!				the device. 			 	
//! \return		TRUE, or FALSE in case of error
//-----------------------------------------------------------------------------
void SPIWriteCharADS7843 (T_SPI_OPEN_STRUCTURE* pOpenContext,UCHAR ucCmd, WORD *pBufOut)
{

	DWORD dwBitIndex = 0;
	UCHAR dwBit = 0;
	int i = 0;

	// Chip select CS#
	AT91F_PIO_ClearOutput(pOpenContext->pSPIInfo->pCSPio, 	AT91C_PA28_NPCS02);

	// Send command
	for (i = 0; i <= 7; i ++)
	{
		// Data MSB first
		dwBit = (ucCmd & (1 << (7-i))) >> (7-i);

		if (dwBit)
		{
			// "1"
			AT91F_PIO_SetOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA1_MOSI0);
		}
		else
		{
			// "0"
			AT91F_PIO_ClearOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA1_MOSI0);
		}
		
		// Clock 
		// Data acquisition on rising edge
		tempo();
		AT91F_PIO_SetOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA2_SPCK0);
		tempo();
		AT91F_PIO_ClearOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA2_SPCK0);
	}

	// After sending the command the MOSI signal must stay to low
	AT91F_PIO_ClearOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA1_MOSI0);

	tempo ();

	// Read conversion data
	(*pBufOut) = 0;
	for (i = 0; i <= 11;)
	{
		//Sleep(2);
		tempo();
		AT91F_PIO_SetOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA2_SPCK0);

		// Read data when busy done
		if (!AT91F_PIO_IsInputSet (pOpenContext->pSPIInfo->pCSPio, AT91C_PIO_PA11))
		{
			(*pBufOut) = ((*pBufOut) << 1) | (AT91F_PIO_IsInputSet (pOpenContext->pSPIInfo->pCSPio, AT91C_PA0_MISO0) ? 1 : 0);
			i++;
		}

		tempo();
		AT91F_PIO_ClearOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA2_SPCK0);
	}

	tempo();

	//* Chip unselect
	AT91F_PIO_SetOutput(pOpenContext->pSPIInfo->pCSPio, 	AT91C_PA28_NPCS02);

	DEBUGMSG(DEBUGSPI9261EK, (TEXT("SPIDriver: -ADS7843_Transaction Data(recu=%d) \r\n"), *pBufOut));


}

//-----------------------------------------------------------------------------
//!
//! \brief		This function performs the SPI transaction for ADS7843
//!
//! \param		pOpenContext		SPI Device	
//! \param		ucCmd	: Command to send to ADS7843			
//! \param		pBufOut : Pointer to the buffer used to transfer the output data from 
//!				the device. 
//! \param		dwLenOut : Maximum number of bytes in the buffer specified by pBufOut. 	
//! \return		TRUE, or FALSE in case of error
//-----------------------------------------------------------------------------
STATIC BOOL
DoSPITransactionADS7843(T_SPI_OPEN_STRUCTURE* pOpenContext,UCHAR ucCmd, WORD* pBufOut, DWORD dwLenOut)
{
	AT91PS_SPI  pSPI = pOpenContext->pSPIInfo->pSPIControllerInfo->pSPI;
	T_SPI_CONTROLLER_STRUCTURE* pSPIControllerInfo = pOpenContext->pSPIInfo->pSPIControllerInfo;

	// The Transaction is protected by a critical section
	// we must get the SPI controller for ourselves.
	EnterCriticalSection(&pSPIControllerInfo->csSPIAccess);
	
	DEBUGMSG(DEBUGSPI9261EK, (TEXT("SPIDriver: +ADS7843_Transaction")));
	
	// Manual transaction

	// MOSI is on PIO_PA1
	AT91F_PIO_CfgOutput (pOpenContext->pSPIInfo->pCSPio, AT91C_PA1_MOSI0);
	AT91F_PIO_ClearOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA1_MOSI0);
	// SPCK is on PIO_PA2
	AT91F_PIO_CfgOutput (pOpenContext->pSPIInfo->pCSPio, AT91C_PA2_SPCK0);
	AT91F_PIO_ClearOutput(pOpenContext->pSPIInfo->pCSPio, AT91C_PA2_SPCK0);

	AT91F_PIO_CfgOutput (pOpenContext->pSPIInfo->pCSPio, 	AT91C_PA28_NPCS02);
	AT91F_PIO_SetOutput(pOpenContext->pSPIInfo->pCSPio, 	AT91C_PA28_NPCS02);
	
	// Input signals
	AT91F_PIO_CfgInput (pOpenContext->pSPIInfo->pCSPio, AT91C_PA0_MISO0);
	AT91F_PIO_CfgInput (pOpenContext->pSPIInfo->pCSPio, AT91C_PIO_PA11);

	// Write Command
	SPIWriteCharADS7843 (pOpenContext,ucCmd,pBufOut);

	// Re enable MOSI (PIO_PA1) as peripheral output
	AT91F_PIO_CfgPeriph (pOpenContext->pSPIInfo->pCSPio, AT91C_PA0_MISO0 | AT91C_PA1_MOSI0 | AT91C_PA2_SPCK0, 0);

	DEBUGMSG(DEBUGSPI9261EK, (TEXT("SPIDriver: -ADS7843_Transaction")));

	//We can now release the access to the SPI controller
	LeaveCriticalSection(&pSPIControllerInfo->csSPIAccess);

	return TRUE;
}


STATIC BOOL 
ADS7843_Read8Bits (UCHAR ucCmd, INT *pDAVVal)
{
	BOOL bRet = FALSE;
	DWORD dwNbBytes = 0;
	WORD wData = 0;
	
    // HWSPIControllerBoardSpecificPIOInitialization
	g_pPIOA->PIO_ASR = AT91C_PA0_MISO0 | AT91C_PA1_MOSI0 | AT91C_PA2_SPCK0 | AT91C_PA3_NPCS00 /*| AT91C_PA4_NPCS01 | AT91C_PA5_NPCS02 | AT91C_PA6_NPCS03*/; 
	g_pPIOA->PIO_BSR = AT91C_PA29_NPCS03;
	g_pPIOA->PIO_PDR = AT91C_PA0_MISO0 | AT91C_PA1_MOSI0 | AT91C_PA2_SPCK0 | AT91C_PA3_NPCS00 | AT91C_PA29_NPCS03/*| AT91C_PA4_NPCS01 | AT91C_PA5_NPCS02 | AT91C_PA6_NPCS03*/;		


    DoSPITransactionADS7843((T_SPI_OPEN_STRUCTURE *)hOpenContext,*((UCHAR *)pBufIn), (WORD*)pBufOut,dwLenOut);



	if (ghTouchScreenSPIDriver == NULL)
		return FALSE;

	
	bRet = DeviceIoControl(
    					ghTouchScreenSPIDriver,		//HANDLE hDevice, 
    					IOCTL_SPI_ADS7843,			//DWORD dwIoControlCode, 
    					&ucCmd,                 	//LPVOID lpInBuffer, 
    					sizeof(ucCmd),          	//DWORD nInBufferSize, 
    					&wData,						//LPVOID lpOutBuffer, 
    					sizeof(wData),				//DWORD nOutBufferSize, 
    					&dwNbBytes,					//LPDWORD lpBytesReturned, 
    					NULL						//LPOVERLAPPED lpOverlapped
    					);	

	if (bRet && dwNbBytes == sizeof(wData))
		*pDAVVal = (INT)wData;
	else 
		bRet = FALSE;
		
	return bRet;
}

PUBLIC BOOL
ADS7843_ReadXY(INT *pUnCalX, INT *pUnCalY)
{
	BOOL bRet;

	// Read X
	bRet = ADS7843_Read8Bits ((ADS7843_READ | ADS7843_X), pUnCalX);

	// Read Y
	return bRet && ADS7843_Read8Bits ((ADS7843_READ | ADS7843_Y), pUnCalY);
}
